package net.wanji.web.service.scheme.impl;

import cn.hutool.core.collection.CollectionUtil;
import net.wanji.web.dto.CrossIdDTO;
import net.wanji.web.dto.RunningPlanDTO;
import net.wanji.web.mapper.scheme.CrossPlanMapper;
import net.wanji.web.mapper.scheme.CrossSchedulesMapper;
import net.wanji.web.mapper.scheme.CrossSchedulesPlanMapper;
import net.wanji.web.mapper.scheme.CrossSchemeMapper;
import net.wanji.web.mapper.scheme.CrossSectionMapper;
import net.wanji.web.po.scheme.CrossPlanPO;
import net.wanji.web.po.scheme.CrossSchedulesPO;
import net.wanji.web.po.scheme.CrossSchedulesPlanPO;
import net.wanji.web.po.scheme.CrossSchemePO;
import net.wanji.web.po.scheme.CrossSectionPO;
import net.wanji.web.service.scheme.RunningPlanService;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/1/28 16:41
 */
@Service
public class RunningPlanServiceImpl implements RunningPlanService {
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final CrossSchedulesPlanMapper crossSchedulesPlanMapper;

    public RunningPlanServiceImpl(CrossPlanMapper crossPlanMapper, CrossSchemeMapper crossSchemeMapper,
                                  CrossSectionMapper crossSectionMapper, CrossSchedulesMapper crossSchedulesMapper,
                                  CrossSchedulesPlanMapper crossSchedulesPlanMapper) {
        this.crossPlanMapper = crossPlanMapper;
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossSectionMapper = crossSectionMapper;
        this.crossSchedulesMapper = crossSchedulesMapper;
        this.crossSchedulesPlanMapper = crossSchedulesPlanMapper;
    }

    @Override
    @Transactional
    public void saveRunningPlan(RunningPlanDTO runningPlanDTO) throws ParseException {
        String crossId = runningPlanDTO.getCrossId();
        // 清空之前计划
        crossPlanMapper.deleteByCrossId(crossId);
        crossSectionMapper.deleteByCrossId(crossId);
        crossSchedulesMapper.deleteByCrossId(crossId);
        crossSchedulesPlanMapper.deleteByCrossId(crossId);
        // 保存计划数据
        saveCrossPlan(runningPlanDTO, crossId);
        // 保存调度数据
        saveCrossSchedules(runningPlanDTO, crossId);
    }

    @Override
    public RunningPlanDTO listRunningPlan(CrossIdDTO crossIdDTO) {
        RunningPlanDTO runningPlanDTO = new RunningPlanDTO();
        String crossId = crossIdDTO.getCrossId();
        runningPlanDTO.setCrossId(crossId);
        // 构造dailyPlanList
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = buildDailyPlanList(crossId);
        runningPlanDTO.setDailyPlanList(dailyPlanList);
        // 构造schedulesPlanList
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = buildSchedulesPlanList(crossId);
        runningPlanDTO.setSchedulesPlanList(schedulesPlanList);

        return runningPlanDTO;
    }

    private List<RunningPlanDTO.SchedulesPlanListElement> buildSchedulesPlanList(String crossId) {
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = new ArrayList<>();
        List<CrossSchedulesPO> crossSchedulesPOList = crossSchedulesMapper.selectByCrossId(crossId);
        for (CrossSchedulesPO crossSchedulesPO : crossSchedulesPOList) {
            String scheduleNo = crossSchedulesPO.getScheduleNo();
            String name = crossSchedulesPO.getName();
            Integer crossSchedulesPOId = crossSchedulesPO.getId();
            RunningPlanDTO.SchedulesPlanListElement schedulesPlan = new RunningPlanDTO.SchedulesPlanListElement();
            schedulesPlan.setScheduleNo(scheduleNo);
            schedulesPlan.setName(name);
            schedulesPlan.setId(crossSchedulesPOId);
            // 构造execList
            List<RunningPlanDTO.ExecListElement> execList = buildExecList(crossId, scheduleNo);
            schedulesPlan.setExecList(execList);
            schedulesPlanList.add(schedulesPlan);
        }
        return schedulesPlanList;
    }

    private List<RunningPlanDTO.ExecListElement> buildExecList(String crossId, String scheduleNo) {
        List<RunningPlanDTO.ExecListElement> execList = new ArrayList<>();
        // 获取调度ID
        CrossSchedulesPO schedulesPO = crossSchedulesMapper.selectByCrossIdAndScheduleNo(crossId, scheduleNo);
        Integer schedulesId = schedulesPO.getId();
        // 根据路口ID、调度ID和计划ID查询调度计划关系
        List<CrossSchedulesPlanPO> crossSchedulesPlanPOList =
                crossSchedulesPlanMapper.selectByCrossIdAndSchedulesId(crossId, schedulesId);
        Map<Integer, List<CrossSchedulesPlanPO>> collectMap = crossSchedulesPlanPOList.stream()
                .collect(Collectors.groupingBy(CrossSchedulesPlanPO::getPlanId));
        for (Map.Entry<Integer, List<CrossSchedulesPlanPO>> entry : collectMap.entrySet()) {
            RunningPlanDTO.ExecListElement execListElement = new RunningPlanDTO.ExecListElement();
            List<Integer> week = new ArrayList<>();
            List<String> specialDateList = new ArrayList<>();
            Integer planId = entry.getKey();
            List<CrossSchedulesPlanPO> secondList = entry.getValue();
            for (CrossSchedulesPlanPO crossSchedulesPlanPO : secondList) {
                Integer day = crossSchedulesPlanPO.getWeek();
                if (day == 0) { // 特殊日期
                    Date specialDate = crossSchedulesPlanPO.getSpecialDate();
                    specialDateList.add(sdf.format(specialDate));
                } else {
                    week.add(day);
                }
            }
            execListElement.setWeek(week);
            execListElement.setSpecialDateList(specialDateList);
            CrossPlanPO crossPlanPO = crossPlanMapper.selectById(planId);
            execListElement.setPlanName(crossPlanPO.getName());
            execList.add(execListElement);
        }
        return execList;
    }

    private List<RunningPlanDTO.DailyPlanListElement> buildDailyPlanList(String crossId) {
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = new ArrayList<>();
        dailyPlanList = crossPlanMapper.selectDailyPlanList(crossId);
        if (CollectionUtil.isEmpty(dailyPlanList)) {
            List<CrossPlanPO> crossPlanPOList = crossPlanMapper.selectByCrossId(crossId);
            if (CollectionUtil.isNotEmpty(crossPlanPOList)) {
                for (CrossPlanPO crossPlanPO : crossPlanPOList) {
                    RunningPlanDTO.DailyPlanListElement dailyPlan = new RunningPlanDTO.DailyPlanListElement();
                    dailyPlan.setPlanNo(crossPlanPO.getPlanNo());
                    dailyPlan.setName(crossPlanPO.getName());
                    dailyPlan.setTimeList(new ArrayList<>());
                    dailyPlanList.add(dailyPlan);
                }
            }
        } else {
            for (RunningPlanDTO.DailyPlanListElement dailyPlan : dailyPlanList) {
                List<RunningPlanDTO.TimeListElement> timeList = dailyPlan.getTimeList();
                for (RunningPlanDTO.TimeListElement timeListElement : timeList) {
                    String[] startEndTime = new String[2];
                    String startTime = timeListElement.getStartTime();
                    String endTime = timeListElement.getEndTime();
                    startEndTime[0] = startTime;
                    startEndTime[1] = endTime;
                    timeListElement.setStartEndTime(startEndTime);
                }
            }
        }
        return dailyPlanList;
    }

    private void saveCrossSchedules(RunningPlanDTO runningPlanDTO, String crossId) throws ParseException {
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = runningPlanDTO.getSchedulesPlanList();
        for (RunningPlanDTO.SchedulesPlanListElement schedulesPlan : schedulesPlanList) {
            // 更新调度表获取调度ID
            CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
            crossSchedulesPO.setScheduleNo(schedulesPlan.getScheduleNo());
            crossSchedulesPO.setName(schedulesPlan.getName());
            crossSchedulesPO.setStatus(0); // 新增调度状态为未执行
            crossSchedulesPO.setCrossId(crossId);
            crossSchedulesMapper.insertOne(crossSchedulesPO);
            Integer crossSchedulesId = crossSchedulesPO.getId();
            // 遍历执行的日计划列表
            List<RunningPlanDTO.ExecListElement> execList = schedulesPlan.getExecList();
            for (RunningPlanDTO.ExecListElement execListElement : execList) {
                // 根据路口ID和计划名获取计划ID
                String planName = execListElement.getPlanName();
                CrossPlanPO crossPlanPO = crossPlanMapper.selectByCrossIdAndName(crossId, planName);
                if (crossPlanPO != null) {
                    Integer planId = crossPlanPO.getId();
                    // 更新调度计划关系表
                    updateCrossSchedulesPlan(crossId, execListElement, crossSchedulesId, planId);
                }
            }
        }
    }

    private void updateCrossSchedulesPlan(String crossId, RunningPlanDTO.ExecListElement schedulesPlan,
                                          Integer crossSchedulesId, Integer planId) throws ParseException {
        List<Integer> week = schedulesPlan.getWeek();
        List<String> specialDateList = schedulesPlan.getSpecialDateList();
        if (CollectionUtil.isNotEmpty(specialDateList)) {
            for (String date : specialDateList) {
                CrossSchedulesPlanPO crossSchedulesPlanPO = initData(crossId, crossSchedulesId, planId);
                crossSchedulesPlanPO.setSpecialDate(sdf.parse(date));
                crossSchedulesPlanPO.setWeek(0);
                crossSchedulesPlanMapper.insertOne(crossSchedulesPlanPO);
            }
        }
        if (CollectionUtil.isNotEmpty(week)) {
            for (Integer day : week) {
                CrossSchedulesPlanPO crossSchedulesPlanPO = initData(crossId, crossSchedulesId, planId);
                crossSchedulesPlanPO.setSpecialDate(new Date(0));
                crossSchedulesPlanPO.setWeek(day);
                crossSchedulesPlanMapper.insertOne(crossSchedulesPlanPO);
            }
        }
    }

    @NotNull
    private static CrossSchedulesPlanPO initData(String crossId, Integer crossSchedulesId, Integer planId) {
        CrossSchedulesPlanPO crossSchedulesPlanPO = new CrossSchedulesPlanPO();
        crossSchedulesPlanPO.setCrossId(crossId);
        crossSchedulesPlanPO.setSchedulesId(crossSchedulesId);
        crossSchedulesPlanPO.setPlanId(planId);
        return crossSchedulesPlanPO;
    }

    private void saveCrossPlan(RunningPlanDTO runningPlanDTO, String crossId) {
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = runningPlanDTO.getDailyPlanList();
        for (RunningPlanDTO.DailyPlanListElement dailyPlan : dailyPlanList) {
            // 更新计划表获取计划ID
            CrossPlanPO crossPlanPO = new CrossPlanPO();
            String planNo = dailyPlan.getPlanNo();
            try {
                Integer.parseInt(planNo);
            } catch (Exception e) {
                throw new RuntimeException("计划编号必须为数字");
            }
            crossPlanPO.setPlanNo(planNo);
            crossPlanPO.setName(dailyPlan.getName());
            crossPlanPO.setCrossId(crossId);
            crossPlanMapper.insertOne(crossPlanPO);
            Integer crossPlanId = crossPlanPO.getId();
            List<RunningPlanDTO.TimeListElement> timeList = dailyPlan.getTimeList();
            for (RunningPlanDTO.TimeListElement time : timeList) {
                // 根据路口ID和方案名获取方案ID
                String schemeName = time.getSchemeName();
                CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeName(crossId, schemeName);
                Integer schemeId = crossSchemePO.getId();
                // 更新时段表
                updateCrossSection(crossId, crossPlanId, schemeId, time);
            }
        }
    }

    private void updateCrossSection(String crossId, Integer crossPlanId, Integer schemeId,
                                    RunningPlanDTO.TimeListElement time) {
        CrossSectionPO crossSectionPO = new CrossSectionPO();
        String[] startEndTime = time.getStartEndTime();
        String startTime = startEndTime[0];
        String endTime = startEndTime[1];
        String s = startTime + endTime;
        String sectionNo = s.replaceAll(":", "");
        crossSectionPO.setSectionNo(sectionNo);
        crossSectionPO.setStartTime(startTime);
        crossSectionPO.setEndTime(endTime);
        crossSectionPO.setCrossId(crossId);
        crossSectionPO.setPlanId(crossPlanId);
        crossSectionPO.setControlMode(1); // todo 暂时只有定周期
        crossSectionPO.setSchemeId(schemeId);
        crossSectionMapper.insertOne(crossSectionPO);
    }
}
